# ngx-lib

ngx-lib is an Angular 5+ library providing foundational functionality to Cleaveland/Price applications.
Use of the library requires some simple configuration of the module(s) that you plan to consume in your application.

## Installation

To install this library, run:

```bash
$ npm install @cleavelandprice/ngx-lib --save
```

## Consuming ngx-lib

After installing ngx-lib, import the module(s) that you need in your `AppModule` file:

```typescript
import { BrowserModule } from '@angular/platform-browser';
import { HttpClientModule } from '@angular/common/http'; // Required by ngx-lib modules that call web services
import { NgModule } from '@angular/core';
import { AppComponent } from './app.component';

// Cleaveland/Price desired module(s)
import {
  AuthenticationModule,
  AuthenticationServiceConfig,
  DialogModule,
  EmailModule,
  EmailServiceConfig,
  EmployeeModule,
  SharePointModule
} from '@cleavelandprice/ngx-lib';

// Define the base URL for C/P API endpoints
// Definining this prefix here makes it easier to define the module-specific Urls
// i.e. The Email module will utilize a different endpoint than the SharePoint module
const webServices = 'http://SomeServer/api';

// Define a configuration for the Email module
const emailConfig = {
  emailApiUrl: `${webServices}/email`,
  server: 'EmailServerAddress',
  smtpPort: 25,
  sender: {
    displayName: 'IT Application Process',
    emailAddress: 'address@domain.com',
    userName: 'emailuser',
    password: 'emailpassword'
  }
};

@NgModule({
  declarations: [
    AppComponent
  ],
  imports: [
    BrowserModule,

    // Import the desired module(s)
    // For the Email module, pass in the configuration that we defined above
    // At this point, all of the functionality exported by each module will be available throughout this app
    // This includes services, components, pipes, classes, etc.
    DialogModule,
    SharePointModule.forRoot({ sharePointApiUrl: `${webServices}/sharepoint` }),
    AuthenticationModule.forRoot({ authenticationApiUrl: `${webServices}/authenticate` }),
    EmailModule.forRoot(emailConfig),
    EmployeeModule.forRoot({ employeeApiUrl: `${webServices}/users` })
  ],
  providers: [],
  bootstrap: [AppComponent]
})
export class AppModule { }
```

Once modules are imported, you can use their components, directives, services, and pipes in your Angular application:

```xml
<!-- app.component.html example -->
<nav>
  <cp-login></cp-login>
</nav>
```
---
### Using AuthenticationModule
The Authentication module provides you with classes and a service that allows you to perform Active/Directory authentication.  You can tap into this functionality "manually" - meaning that you construct the interface to acquire user credentials and pass it into the service call, and you have complete control over the user interface before, during, and after authentication.

Or, you may choose to simply use the `cp-login` component, which performs all of the heavy lifting for you.  By simply including the `cp-login` tag into your HTML markup, the Authentication module provides its own login interface, built with Angular Material components, and displays the user's photo after authentication.  This method requires no manual coding or user interface construction.  It's well suited as a drop-in component for a toolbar.

Regardless of which of the two approaches you decide to take, the Authentication module will handle management of the authentication token, saving it in the browser's Local Storage and automatically retrieving it the next time the user visits the page - preventing them from having to re-authenticate each time.

The Authentication module also the developer to (optionally) specify an Active Directory group that a user must belong to in order to be considered an administrator of the application.  When the web service returns the authentication token an Admin flag will be set to True or False depending on whether or not the user belongs to the group specified.  It is then up to the developer to customize the user's experience within the app based on this value.  In some cases, it may even be desireable for the app to restrict login to only members of the specified group.  In such a case, you would perform the authentication (which would succeed), and if the user isn't an Admin (because they don't belong to the group), you would call the `logout()` method of the AuthenticationService and display a message informing them that they can't login without the required group membership.

#### Using the Authentication service (manual login/logout)
```typescript
import { AuthenticationService } from '@cleavelandprice/ngx-lib';

@Component({
  selector: 'app-authentication',
  template: `
    <input type="text" [(ngModel)]="username">
    <input type="text" [(ngModel)]="password">
    <img *ngIf="authenticationService.authenticated && authenticationService.authenticatedUser.photoUrl"
      [src]="authenticationService.authenticatedUser.photoUrl" />
    <button (click)="login()">Login</button>
    <button (click)="logout()">Logout</button>
    `
})
export class AuthenticationComponent implements OnInit {
  username: string;
  password: string;
  adminGroup = 'App Admins'; // Optional AD group specifying who is an admin of this app
  requiredGroup: boolean; // Optional login restriction

  constructor(public authenticationService: AuthenticationService) {}

  ngOnInit(): void {
    // If we're requiring the user to be in a certain group to login, then check it after login
    // If not a valid member, then log them out
    this.authenticationService.authenticatedChanged
      .subscribe(() => {
        if (this.authenticationService.authenticated) {
          if (this.adminGroup && this.requiredGroup) {
            if (!this.authenticationService.authenticatedUser.admin) {
              this.logout();
              alert(`Sorry bro, you're not a member of '${this.adminGroup}'`);
              return;
            }
          }
          alert('Logged in');
        } else {
          alert('Logged out');
        }
      });
  }

  login() {
    if (this.username && this.password) {
      this.authenticationService.login(this.username, this.password, this.adminGroup);
    }
  }

  logout() {
    this.authenticationService.logout();
  }
}
```
---
### Using DialogModule
The Dialog module provides a simple way to display 3 types of dialog boxes built with Angular Material components:

1. **Message Box** displays a message but does not provide a way for the user to type input.  The only button displayed is an OK button.
2. **Confirmation Box** displays a simple question for the user and returns a true/false value indicating which button they selected.  The buttons can be of the following types: `Ok/Cancel` (default option), `Yes/No`, `True/False`.  Regardless of which option is selected by the developer, it will always return a true/false value.
3. **Input Box** displays a dialog box with an input field that the user can type into.  The user can then click Ok, or Cancel.

All three types of dialogs allow the developer to specify the Title and the Message to be displayed.  Additionally, the developer may optionally provide a message to be displayed via a checkbox.  If the `checkboxMessage` is provided, then the dialog box will include a checkbox at the bottom and the value (true/false) will be returned to the calling code.  This also applies to dialogs of type Message Box - which otherwise doesn't return a value.  Once the value is returned, it's up to the developer to decide what to do with the checkbox value.  Typically, this is used for options like *Don't show me this message in the future*, etc.

To prevent the user from closing the dialog without responding (i.e. clicking outside of the dialog), simply pass in the optional `disableClose` parameter provided by Angular Material (this is native functionality provided by Angular).

```typescript
import { MatDialog } from '@angular/material/dialog';
import {
  ConfirmationDialogComponent,
  ConfirmationDialogMode,
  InputDialogComponent,
  MessageDialogComponent
} from '@cleavelandprice/ngx-lib';
@Component({
  selector: 'app-dialogs',
  template: `
    <input type="text" [(ngModel)]="title">
    <input type="text" [(ngModel)]="message">
    <input type="text" [(ngModel)]="checkboxMessage">
    <input type="text" [(ngModel)]="placeholder">
  `
})
export class DialogsComponent {
  confirmationMode = ConfirmationDialogMode.OkCancel;
  title: string;
  message: string;
  checkboxMessage: string; // Optional
  placeholder: string; // For input box
  disableClose = false; // Optional Angular Material option to specify modal

  constructor(public dialog: MatDialog) { }

  messageBox(): void {
    this.dialog
      .open(MessageDialogComponent, {
        data: {
          title: this.title,
          message: this.message,
          checkboxMessage: this.checkboxMessage
        },
        disableClose: this.disableClose
      })
      .afterClosed()
      .subscribe(response => {
        if (this.checkboxMessage) {
          console.log(response);
        }
      });
  }

  confirmationBox(): void {
    this.dialog
      .open(ConfirmationDialogComponent, {
        data: {
          title: this.title,
          message: this.message,
          checkboxMessage: this.checkboxMessage,
          mode: this.confirmationMode
        },
        disableClose: this.disableClose
      })
      .afterClosed()
      .subscribe(response => console.log(response));
  }

  inputBox(): void {
    this.dialog
      .open(InputDialogComponent, {
        data: {
          title: this.title,
          message: this.message,
          checkboxMessage: this.checkboxMessage,
          placeholder: this.placeholder
        },
        disableClose: this.disableClose
      })
      .afterClosed()
      .subscribe(response => console.log(response));
  }
}
```
---
### Using EmailModule
The Email module provides a simple mechanism for sending emails via the Cleaveland/Price web services.

```typescript
import { EmailService, Email } from '@cleavelandprice/ngx-lib';

@Component({
  selector: 'app-email',
  template: '<button (click)="sendMail()">Send Email</button>'
})
export class EmailComponent {
  // Inject the EmailService into this component
  constructor(private emailService: EmailService) { }

  sendEmail(): void {
    // Define an email object
    const email: Email = {
      subject: 'Test Email',
      recipients: {
        to: ['recipient@domain.com']
      },
      content: { html: '<html><body>hooray!</body></html>' }
    };

    // Send the email
    this.emailService.send(email).subscribe();
  }
}
```
---
### Using EmployeesModule
---
### Using SharePointModule
---
## License

MIT © [Dan Rullo](mailto:drullo@cleavelandprice.com)

This project was built using the <a href="https://github.com/dherges/ng-packagr">ng-packagr</a> project.
